/* STARTUP(crt0.o) */
/* To generate PIC code use only elf32-m68k and then convert to srec */
OUTPUT_FORMAT(elf32-m68k)
OUTPUT_ARCH(m68k)
SEARCH_DIR(.)
__DYNAMIC  =  0;

/*
 * Setup the memory map of the Khepera
 *
 * The memory map look like this:
 * +--------------------+ <- low memory
 * | .text              |
 * |        _etext      |
 * |        ctor list   | the ctor and dtor lists are for
 * |        dtor list   | C++ support
 * +--------------------+
 * | .data              | initialized data goes here
 * |        _edata      |
 * +--------------------+
 * | .bss               |
 * |        __bss_start | start of bss, cleared by crt0
 * |        _end        | start of heap, used by sbrk()
 * +--------------------+
 * .                    .
 * .                    .
 * .                    .
 * |        __stack     | top of stack
 * +--------------------+
 */
MEMORY
{
  rom     : ORIGIN = 0x000000, LENGTH = 128K
  ramsys  : ORIGIN = 0x100000, LENGTH = 32K
  ramcode : ORIGIN = 0x108000, LENGTH = 992K
}

/*
 * allocate the stack to be at the top of memory, since the stack
 * grows down
 */
PROVIDE (__stack = 0x1FFF00);

PROVIDE (ApplStr = 0);
PROVIDE (HelpStr = 0);

/*
 * Initalize some symbols to be zero so we can reference them in the
 * crt0 without core dumping. These functions are all optional, but
 * we do this so we can have our crt0 always use them if they exist. 
 * This is so BSPs work better when using the crt0 installed with gcc.
 */
PROVIDE (hardware_init_hook = 0);
PROVIDE (software_init_hook = 0);

/*
 * Module identity
 * 	"FU" stand alone executable function code
 * 	"PR" SerCom protocol function code
 * 	"TO" SerCom tool function code
 * 	"BI" Bios
 * 	"ST" Start-up code
 * 	"LI" Lib
 * 	"DG" Debug
 * 	"TA" Proc
 */
PROVIDE (idmodule = 0x46555541);

/*
 * File type :
 * 	0 visible in help
 * 	1 executable from SerCom
 * 	2 executable from run
 * 	3 confidential
 */
PROVIDE (fltype = 1);

/*
 * Output type :
 *      0 serial output
 *      1 communication output
 *      2 usart output
 */
PROVIDE (serial_io = 2);

/*
 * stick everything in ram (of course)
 */
SECTIONS
{

  /* Needed. Without, PIC don't work properly */
  .rela.got      : 
  { 
   *(.rela.got)
  } > ramcode

  _start = .;
  /* Code */
  .text :
  {
    *(.text)
    *(.stub)
    *(.gnu.warning)
    *(.gnu.linkonce.t*)
  } > ramcode

  /* Read Only initialized data */
  .rodata BLOCK (0x4) :
  {
    *(.rodata)
    *(.gnu.linkonce.r*)
  } > ramcode

  .gcc_except_table BLOCK (0x4) :
  {
    *(.gcc_except_table) 
  } > ramcode

  /* Writable initialized data */
  .data :
  {
    *(.data)
    *(.gnu.linkonce.d*)      
    CONSTRUCTORS
  } > ramcode

  /* Constructors (C++) */
  .ctors BLOCK (0x4) :
  {
    /* Constructors */
     __CTOR_LIST__ = .;
    *(.ctors)
    __CTOR_END__ = .;
  } > ramcode

  /* Destructors (C++) */
  .dtors BLOCK (0x4) :
  {
    /* Destructors */
    __DTOR_LIST__ = .;
    *(.dtors)
    __DTOR_END__ = .;
  } > ramcode

  /* Global offset table for position independent code generation */
  .got BLOCK(0x4) :
  {
    *(.got.plt)
    *(.got)
  } > ramcode

  /* Writable Uninitialized data */
  .bss BLOCK (0x4) :
  {
    __bss_start = . ;
    *(.dynbss)
    *(.bss)
    *(COMMON)
    __bss_end = . ;
  } > ramcode
  _end =  ALIGN (0x8);
  /* DWARF debug sections.
     Symbols in the DWARF debugging sections are relative to the beginning
     of the section so we begin them at 0.  */
  /* DWARF 1 */
  .debug          0 : { *(.debug) }
  .line           0 : { *(.line) }
  /* GNU DWARF 1 extensions */
  .debug_srcinfo  0 : { *(.debug_srcinfo) }
  .debug_sfnames  0 : { *(.debug_sfnames) }
  /* DWARF 1.1 and DWARF 2 */
  .debug_aranges  0 : { *(.debug_aranges) }
  .debug_pubnames 0 : { *(.debug_pubnames) }
  /* DWARF 2 */
  .debug_info     0 : { *(.debug_info) }
  .debug_abbrev   0 : { *(.debug_abbrev) }
  .debug_line     0 : { *(.debug_line) }
  .debug_frame    0 : { *(.debug_frame) }
  .debug_str      0 : { *(.debug_str) }
  .debug_loc      0 : { *(.debug_loc) }
  .debug_macinfo  0 : { *(.debug_macinfo) }
  /* SGI/MIPS DWARF 2 extensions */
  .debug_weaknames 0 : { *(.debug_weaknames) }
  .debug_funcnames 0 : { *(.debug_funcnames) }
  .debug_typenames 0 : { *(.debug_typenames) }
  .debug_varnames  0 : { *(.debug_varnames) }


  /* Computation of symbols used for the KTEAM Header */
  ptrrom = ADDR(.text);
  lnrom = _end - _start;
  ptrram = _end;
  /* End of memory - size of stack - end of program */
  /* Also used in sbrk (i.e. malloc) */
  lnram = __stack - 1K - _end;
 
}
