/*
   ; Fonctionnement par Interruption(tests)
   ; ======================================
   ;------------------------------------------------------------------------
   ; Author:    Olivier Carmona.        08/02/99
   ;
   ; Project:   Vision Sphere 
   ; Goal: Interruption generation using the CTM (FCSM12 submodule)
   ; Note: MCSM2 module is not working properly 
   ; 
   ;------------------------------------------------------------------------
*/


#include <sys/kos.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <math.h>


/* Variable that must be the first one */
static unsigned long vIDProcess[1];

/* Process 0
 * ^^^^^^^^^
 */

void enable_irq_mcsm(void);
void enable_irq_fcsm(void);
void disable_irq_mcsm(void);
void disable_irq_fcsm(void);

void loop_mcsm(void);
void loop_fcsm(void);

static unsigned int typical_time = 0;
static unsigned char cpt1 = 0, cpt2 = 0;
static unsigned char cpt3 = 0, cpt4 = 0;

asm("
loop_fcsm: 
        movem.l a0-a6/d0-d7,-(sp)
	jsr loop_fcsmC(pc)
        movem.l (sp)+,a0-a6/d0-d7
	rte
");

void loop_fcsmC(void)
{
 unsigned int last_ticcount;
 static unsigned int ticcount;

 disable_irq_fcsm();
 if (cpt3++ == 0) 
 {
  last_ticcount = ticcount;
  ticcount = tim_get_ticcount();
  typical_time = ticcount - last_ticcount;
  var_change_led(2+(cpt4++)%2);
 }
}

asm("
loop_mcsm: 
        movem.l a0-a6/d0-d7,-(sp)
	jsr loop_mcsmC(pc)
        movem.l (sp)+,a0-a6/d0-d7
	rte
");

void loop_mcsmC(void)
{
 unsigned int last_ticcount;
 static unsigned int ticcount;

 disable_irq_mcsm();
 if (cpt1++ == 0) 
 {
  last_ticcount = ticcount;
  ticcount = tim_get_ticcount();
  typical_time = ticcount - last_ticcount;
  var_change_led((cpt2++)%2);
 }
}

static void
process_0 ()
{
 /* Interrupt Number = Default Base for CTM (0xC0) + Module Number for MCSM (0x2)*/
 var_set_exception(loop_mcsm, 0xC2);

 /* Interrupt Number = Default Base for CTM (0xC0) + Module Number for FCSM (0xC)*/
 var_set_exception(loop_fcsm, 0xCC);
 
 /* Activate FCSM interrupt */
 enable_irq_fcsm();
 
 for (;;)
  {
   sleep(10);
   printf("Time %f ms (%d,%d)\r\n",(float)typical_time/256,cpt1,cpt3);
  }
}


/* MAIN
 * ====
 *
 * - Initialize the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

main (void)
{
 int32 status;
 static char prName_0[] = "Interrupt test, OC-97          Rev. 1.00\r\n";

 status = install_task (prName_0, 1000, process_0);
 if (status == -1)
   exit (0);
 vIDProcess[0] = (unsigned long) status;
}
