/*
   ; Implement a new protocol to control the Khepera
   ; ===============================================

   ;------------------------------------------------------------------------
   ; Author:    Franzi Edo.     The 03/04/98
   ; Modifs:
   ;
   ; Project:   KHEPERA
   ; Goal:              This PRGM launches 2 processes.
   ;                    Process 0: "alive" it toggles the LED 0 every 500 ms.
   ;                    Process 1: it implements a simple protocol to controle Khepera.
   ;
   ; This module is (c) of Edoardo Franzi 1992-1998
   ; franzi@k-team.com
   ;
   ;------------------------------------------------------------------------
 */

#include	<sys/kos.h>
#include 	<stdlib.h>
#include 	<stdio.h>

static int32 vIDProcess[2];

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */

static void
process_0 ()
{

  for (;;)
    {
      tim_suspend_task (500);
      var_change_led (0);
    }
}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * This process implements a simple protocol to control Khepera.
 * It waits until a command line is available (an ASCII string
 * terminated by \n) and it displays it.
 *
 */

#define		LNBUFFER	100

static char getChar (void);

static void
process_1 ()
{
  char bufferIn[LNBUFFER];
  int terminate, i;

/* Flush the channel */
  while ((com_get_status_channel () & 0x2) == 0x2)
    com_receive_byte ();

  for (;;)
    {

/* Flush the channel */

      i = 0;
      terminate = FALSE;
      while (terminate == FALSE)
	{
	  bufferIn[i] = getc (stdin);
	  switch (bufferIn[i])
	    {
	    case '\b':
	      {
		if (i != 0)
		  {
		    i -= 1;
		    bufferIn[i] = 0x0;
		  }
		break;
	      }

	    case '\r':
	    case '\n':
	      {
		if (i != LNBUFFER)
		  {
		    i += 1;
		  }
		bufferIn[i] = 0x0;
		terminate = TRUE;
		break;
	      }

	    default:
	      {
		if (i != LNBUFFER)
		  {
		    i += 1;
		  }
		break;
	      }
	    }
	}

/* Echo ... and action */

      printf ("%s\r", bufferIn);
    }
}

static char
getChar (void)
{
  int32 status;
  uint32 aChar;

  do
    {
      status = com_get_status_channel () & 0x2;
    }
  while (status != 0x2);
  aChar = (uint32) com_receive_byte ();
  return (char) aChar;
}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

int
main (void)
{
  int32 status;

  /*
     --------------------------|------------------------------------------------|
   */

  static char prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
  static char prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";

  com_reset ();
  var_reset ();
  mot_reset ();
  str_reset ();

  status = install_task (prName_0, 800, process_0);
  if (status == -1)
    exit (0);
  vIDProcess[0] = (uint32) status;

  status = install_task (prName_1, 800, process_1);
  if (status == -1)
    exit (0);
  vIDProcess[1] = (uint32) status;

  return 0;
}
