/*
   ; Print the ROM & the BIOS ID
   ; ===========================

   ;------------------------------------------------------------------------
   ; Author:    Franzi Edo.     The 03/04/98
   ; Modifs:
   ;
   ; Project:   KHEPERA
   ; Goal:              This PRGM launches 2 processes.
   ;                    Process 0: "alive" it toggles the LED 0 every 500 ms.
   ;                    Process 1: it displays the versions of ROM & BIOS every 1 s.
   ;
   ;                    - After 10 s a restart is performed!
   ;
   ; This module is (c) of Edoardo Franzi 1992-1998
   ; franzi@k-team.com
   ;
   ;------------------------------------------------------------------------
 */

#include	<sys/kos.h>
#include 	<stdlib.h>
#include 	<stdio.h>

static int32 vIDProcess[2];

/* ID of the module Start-Up                           */
/* The revision of the ROM is contained in this module */

#define KIDCSTUP       ('T'<<24)+('A'<<16)+('S'<<8)+'T'

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */

static void
process_0 ()
{

  for (;;)
    {
      tim_suspend_task (500);
      var_change_led (0);
    }
}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * This process displays the BIOS and ROM version
 * every 1 s.
 *
 */

static void
process_1 ()
{
  int i;
  BIGPTRTABLE *mainPtr;
  HEADERMODULE *modulePtr;
  char stroutIdROM[5], stroutIdBIOS[5];

  for (;;)
    {
      tim_suspend_task (1000);

/* Find the module "Start-Up" */

      mainPtr = (BIGPTRTABLE *) aBigPtr;
      modulePtr = (HEADERMODULE *) (mainPtr->oPtrFrstMod);
      do
	{
	  modulePtr = (HEADERMODULE *) ((uint32) modulePtr + (uint32) modulePtr->oLnCODE);
	}
      while (modulePtr->oIDModule != KIDCSTUP);

      /* The ROM revision */

      i = 0;
      stroutIdROM[i++] = (char) (modulePtr->oRevision >> 24);
      stroutIdROM[i++] = (char) (modulePtr->oRevision >> 16);
      stroutIdROM[i++] = (char) (modulePtr->oRevision >> 8);
      stroutIdROM[i++] = (char) (modulePtr->oRevision);
      stroutIdROM[i++] = (char) (0);

      /* The BIOS revision */

      i = 0;
      stroutIdBIOS[i++] = (char) (bios_get_rev () >> 24);
      stroutIdBIOS[i++] = (char) (bios_get_rev () >> 16);
      stroutIdBIOS[i++] = (char) (bios_get_rev () >> 8);
      stroutIdBIOS[i++] = (char) (bios_get_rev ());
      stroutIdBIOS[i++] = (char) (0);
      printf ("ROM revision = %s, BIOS revision = %s\r\n", stroutIdROM, stroutIdBIOS);
    }
}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

int
main (void)
{
  int32 status;

  /*
     --------------------------|------------------------------------------------|
   */

  static char prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
  static char prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";

  com_reset ();
  var_reset ();

  status = install_task (prName_0, 800, process_0);
  if (status == -1)
    exit (0);
  vIDProcess[0] = (uint32) status;

  status = install_task (prName_1, 800, process_1);
  if (status == -1)
    exit (0);
  vIDProcess[1] = (uint32) status;

/* Wait 10 seconds and perform a restart! */

  tim_suspend_task (10000);
  bios_restart_robot ();

  return 0;
}
