/*
   ; Print the sytem name
   ; ====================

   ;------------------------------------------------------------------------
   ; Author:    Franzi Edo.     The 03/04/98
   ; Modifs:
   ;
   ; Project:   KHEPERA
   ; Goal:              This PRGM launches 2 processes.
   ;                    Process 0: "alive" it toggles the LED 0 every 500 ms.
   ;                    Process 1: it displays the ID of the system every 1 s.
   ;
   ; This module is (c) of Edoardo Franzi 1992-1998
   ; franzi@k-team.com
   ;
   ;------------------------------------------------------------------------
 */

#include	<sys/kos.h>
#include 	<stdlib.h>
#include 	<stdio.h>

static int32 vIDProcess[2];

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */

static void
process_0 ()
{

  for (;;)
    {
      tim_suspend_task (500);
      var_change_led (0);
    }
}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * This process displays the ID of the system
 * every 1 s.
 *
 */

static void
process_1 ()
{
  char *strout;
  char khepera[] = "Khepera system\r\n";
  char KheperaRadio[] = "Khepera Radio base system\r\n";
  char Koala[] = "Koala system\r\n";
  char KoalaCharger[] = "Koala charger system\r\n";
  char Top331[] = "Top331 turret system\r\n";
  char Ext336[] = "Ext336 system\r\n";
  char DETEC[] = "DETEC system\r\n";
  char MotSys[] = "MotSys system\r\n";
  char Man331[] = "Man331 system\r\n";
  char Tribolo[] = "Tribolo system\r\n";
  char Unknown[] = "Unknown system\r\n";
  uint32 family;

  for (;;)
    {
      tim_suspend_task (1000);
      family = bios_get_robot ();
      switch (family)
	{
	case 0:
	  strout = khepera;
	  break;
	case 1:
	  strout = KheperaRadio;
	  break;
	case 2:
	  strout = Koala;
	  break;
	case 3:
	  strout = KoalaCharger;
	  break;
	case 4:
	  strout = Top331;
	  break;
	case 5:
	  strout = Ext336;
	  break;
	case -1:
	  strout = DETEC;
	  break;
	case -2:
	  strout = MotSys;
	  break;
	case -3:
	  strout = Man331;
	  break;
	case -4:
	  strout = Tribolo;
	  break;
	default:
	  strout = Unknown;
	  break;
	}
      printf ("%s", strout);
    }
}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

int
main (void)
{
  int32 status;

  /*
     --------------------------|------------------------------------------------|
   */

  static char prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
  static char prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";

  com_reset ();
  var_reset ();

  status = install_task (prName_0, 800, process_0);
  if (status == -1)
    exit (0);
  vIDProcess[0] = (uint32) status;

  status = install_task (prName_1, 800, process_1);
  if (status == -1)
    exit (0);
  vIDProcess[1] = (uint32) status;

  return 0;
}
