/* ====================================================
 * Approximation of sin(x) in [-pi/2,pi/2]
 * Input / Output Number Representation: 4.28 bits
 * Maximum Error: 10E-6 close to sin(pi/2)
 *
 * Copyright (C) 2000 by K-Team, S.A. All rights reserved.
 *
 * Developed at K-Team S.A by Olivier Carmona.
 *
 * Permission to use, copy, modify, and distribute this
 * software is freely granted, provided that this notice
 * is preserved.
 *
 * There is no warranty for this free software.  If the software is modified
 * by someone else and passed on, we want its recipients to know that
 * what they have is not the original, so that any problems introduced
 * by others will not reflect on the original authors' reputations.
 *
 * ====================================================
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* 
  Scale factor for conversion from double to
  number representation 4.28 bits & conversely
 */
double scale16, invscale16;

long pio2, one;
long sinstage1, sinstep1, sinstep2;
unsigned long sin_lkup1 (unsigned int);
unsigned long sin_lkup2 (unsigned int);

void
init_sina (void)
{
  sinstage1 = (long) ((M_PI / 4) * scale16);
  /* 
    Important: round up!!!
   */
  sinstep1 = (sinstage1 >> 8) + 1;
  sinstep2 = ((pio2 - sinstage1) >> 8) + 1;
}

inline long
sina (long x)
{
  int sign = 0;
  long resultat = 0;
  unsigned long xvalue, resultat_int;
  unsigned char site = 0, interpolation = 0;
  unsigned int site_int;

  if (x < 0)
    sign = 1, xvalue = (-x);
  else
    xvalue = x;

  if (xvalue == pio2)
    return (sign == 0 ? one : -one);

  if (xvalue < sinstage1)
    {
      interpolation = ((xvalue % sinstep1) * 256) / sinstep1;
      site = xvalue / sinstep1;
      site_int = (site << 8) + interpolation;
      resultat_int = sin_lkup1 (site_int);
      resultat = (resultat_int >> 5);
    }
  else if (xvalue < pio2)
    {
      interpolation = (((xvalue - sinstage1) % sinstep2) * 256) / sinstep2;
      site = (xvalue - sinstage1) / sinstep2;
      site_int = (site << 8) + interpolation;
      resultat_int = sin_lkup2 (site_int);
      resultat = (resultat_int >> 5);
    }
  else
    printf ("Error sin(x) out of approximation domain [0,pi/2].\r\n");

  return (sign == 0 ? resultat : -resultat);
}

inline long
cosa (long x)
{
  if (x < 0)
    x = -x;

  return (sina (pio2 - x));
}

int
main (void)
{
 short a;
 long phi_rad;
 long sin_phi;
 long dego2rad;

 /* Initialization of Global Variables */
 scale16 = ((double) 0x7FFFFFFF) / 16;      
 invscale16 = 1/scale16;
 pio2 = (long) ((M_PI / 2) * scale16);
 one = (long) (1.0 * scale16);


 dego2rad = (long) ((2 * M_PI / 360) * scale16);

 init_sina ();

 /*
    Loop does nothing except computing sin and cos 
    and printing one result
  */
  for (;;)
   {
     printf("Enter an angle in degree between 0 and 90: \r\n"); 
     scanf("%hd",&a); 
 
     phi_rad = a * dego2rad;
     sin_phi = sina (phi_rad);
     
     printf ("ANGLE: %d SIN: %f\r\n", a, (double)sin_phi * invscale16);
   }

}
