% ====================================================
% Matlab code to generate a look-up table to 
% approximate the function sin(x) in [0 pi/2].
% Number Representation: 0.32 bits
%
% Copyright (C) 2000 by K-Team, S.A. All rights reserved.
%
% Developed at K-Team S.A by Olivier Carmona.
%
% Permission to use, copy, modify, and distribute this
% software is freely granted, provided that this notice
% is preserved.
%
% There is no warranty for this free software.  If the software is modified
% by someone else and passed on, we want its recipients to know that
% what they have is not the original, so that any problems introduced
% by others will not reflect on the original authors' reputations.
%
% ====================================================


clear all;

%* Definition of the domain */
alfamax = pi/2;
alfastep = pi/4;
deltaalfa = 1E-5;
alfamin = 0;


alfa = alfamin:deltaalfa:alfamax;

%* Calculus of the function */

deltax = sin(alfa);


%* Build the Look Up Table */

intstep1 = (alfastep - alfamin)/256;
disalfa1 = alfamin:intstep1:alfastep;
intstep2 = (alfamax - alfastep)/256;
disalfa2 = alfastep:intstep2:alfamax;
disalfa = [disalfa1(1:256) disalfa2(1:256)];
disalfab = [disalfa1 disalfa2];
bix = round((sin(disalfa))*(2^32 - 1));
bixb = round((sin(disalfab))*(2^32 - 1));
ix = bix/(2^32 - 1);
ixb = bixb/(2^32 - 1);

%* Save the Look Up Table in File */

fid = fopen('sina.i','w');
fprintf(fid,'| Table generated by Matlab\n');
fprintf(fid,'| Source : ~/source/matlab/vision/platform/sina2_demo.m\n');
fprintf(fid,'| Approximation of sin(x) in [0,pi/2]\n');
fprintf(fid,'| Numbers are represented in the whole 32 bits\n');
fprintf(fid,'|\n');
fprintf(fid,'.data\n');
fprintf(fid,'aTabSin1:\n');
fprintf(fid,'\t.long\t0x%8x, 0x%8x, 0x%8x, 0x%8x\n',bixb(1:257));
fprintf(fid,'aTabSin2:\n');
fprintf(fid,'\t.long\t0x%8x, 0x%8x, 0x%8x, 0x%8x\n',bixb(258:514));
fprintf(fid,'\n\n');

%* Interpolation calculus */
 
interx = interp1(disalfa,ix,alfa);
errx = abs(interx-deltax);

%* Display Interpolation Error */
 
figure(1);
clf;
grid on;
title('Interpolation Error in [0 pi/2]');
plot(alfa, errx);
print -djpeg90 sina2.jpg
 

