/*
   ; Switch back to the sercom module
   ; ================================

   ;------------------------------------------------------------------------
   ; Author:    Franzi Edo.     The 03/04/98
   ; Modifs:
   ;
   ; Project:   KHEPERA
   ; Goal:              Demo of a C application.
   ;                    This application shows how to return to the "sercom" module.
   ;
   ; This module is (c) of Edoardo Franzi 1992-1998
   ; franzi@k-team.com
   ;
   ;------------------------------------------------------------------------
 */

#include	<sys/kos.h>
#include 	<stdlib.h>
#include 	<stdio.h>

#define	KIDCSTUP	('T'<<24)+('A'<<16)+('S'<<8)+'T'

uint32 vIDProcess[2];

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */

void
process_0 ()
{

  for (;;)
    {
      tim_suspend_task (500);
      var_change_led (0);
    }
}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * This process tries to send a string on the "COM" channel
 * every 1000 ms.
 * After 15 s the task 1 of this application is killed.
 * A new task (sercom) is launched.
 *
 */

void
process_1 ()
{
  uint32 i;

  BIGPTRTABLE *mainPtr;
  HEADERMODULE *modulePtr;
  PROCESS *processPtr;
  uint32 sizeDATA, *processStack;
  void *processCode;
  char *processText;

  i = 0;
  do
    {
      tim_suspend_task (1000);
      printf ("K-System %ld\r\n", i);
      i++;
    }
  while (i < 5);

/* Terminate; launch the Start-up again (to avoid the reset...) */
/* ------------------------------------------------------------ */

  mainPtr = (BIGPTRTABLE *) aBigPtr;
  modulePtr = (HEADERMODULE *) mainPtr->oPtrFrstMod;
  sizeDATA = (uint32) modulePtr->oPtrDATA;
  do
    {
      if (modulePtr->oIDModule == KIDCSTUP)
	{
	  processPtr = (PROCESS *) ((uint32) & modulePtr->oDescProc + (uint32) modulePtr->oDescProc);
	  processStack = (uint32 *) (sizeDATA + (uint32) processPtr->oPtrProcStack);
	  processText = (char *) ((uint32) & processPtr->oPtrProcText + (uint32) processPtr->oPtrProcText);
	  processCode = (void *) ((uint32) & processPtr->oPtrProcCode + (uint32) processPtr->oPtrProcCode);
	  tim_new_inst_task (processText, processStack, processCode);

/* And now, kill the task 1 */

	  kill_task (vIDProcess[0]);
	  kill_task (vIDProcess[1]);
	}

      sizeDATA += modulePtr->oLnDATA;
      modulePtr = (HEADERMODULE *) ((uint32) modulePtr + (uint32) modulePtr->oLnCODE);
    }
  while (modulePtr->oIDModule != 0xFFFFFFFF);
  printf ("This task does not exist\r\n");
  while (TRUE)
    tim_switch_fast ();
}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

int
main (void)
{
  int32 status;

  /*
     --------------------------|------------------------------------------------|
   */

  static char prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
  static char prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";

  com_reset ();
  var_reset ();

  status = install_task (prName_0, 800, process_0);
  if (status == -1)
    exit (0);
  vIDProcess[0] = (uint32) status;

  status = install_task (prName_1, 800, process_1);
  if (status == -1)
    exit (0);
  vIDProcess[1] = (uint32) status;

  return 0;
}
