/*
; BIOS and COM system call and definitions.
; =========================================

;------------------------------------------------------------------------
; Author:	Franzi Edo.	The 03/04/98
; Modifs:	Olivier Carmona The 05/08/99
;
; Project:	KHEPERA
; Goal:		BIOS and COM system call and definitions.
;
;------------------------------------------------------------------------
*/

#ifndef	_KOS_H
#define	_KOS_H

#include <sys/ktypes.h>

#ifdef  __cplusplus
extern "C" {
#endif

/* Khepriv */
/* ======= */

#define	return_unlock(x)	return(tim_unlock(), x)

/* Private BIOS system calls */
/* ========================= */

extern  void    tim_run_kernel(void);

#define	ABIGPTR			0x400		/* location of a big ptr on some useful ptr */

/* Important structures */
/* ==================== */

typedef	struct	procDesc		PROCDESC;
typedef	struct	process			PROCESS;
typedef	struct	headerModule	HEADERMODULE;
typedef	struct	bigPtrTable		BIGPTRTABLE;

/* Structure of the process descriptor of the microkernel */
/* ------------------------------------------------------ */

struct	procDesc
	{
	uint32		oIDProc;		/* process ID					 */
	uint8		oStaProc;		/* process state				 */
	#define		bPres	 	0	/* process present				 */
	#define		bLock		1	/* process lock					 */

	uint8		oReserve;		/* reserve      				 */
	uint32		oMskEvent;		/* event mask					 */
	uint32		oMskSync;		/* synchro mask					 */
	PROCDESC	*oPtrCurrent;	/* ptr on the belong descriptor	 */
	PROCDESC	*oPtrBack;		/* ptr on the back descriptor	 */
	PROCDESC	*oPtrForward;	/* ptr on the forward descriptor */
	char		*oPtrTextID;	/* ptr on the descriptor ID text */
	uint32		oTStopProc;		/* a wait time					 */
	uint32		*oPtrStkPro;	/* process stack				 */
	};

/* Structure of the process descriptor (pointed by (oDescProc,Ax)) */
/* --------------------------------------------------------------- */

struct	process
	{
	uint32	*oPtrProcCode;		/* ptr on the process code			 */
	uint32	*oPtrProcStack;		/* ptr on the process stack			 */
	char	*oPtrProcText;		/* ptr on the process text ID string */
	};

/* Structure of the header module */
/* ------------------------------ */

struct	headerModule
	{
	uint32	oBRACode;			/* jump to the module												  */
	uint32	oIDModule;			/* ID of the module T T n n											  */
	#define	kIDFunction	'FU'	/*					F U x x ---> stand alone executable function code */
	#define	kIDProtocol	'PR'	/*					P R x x ---> SerCom protocol function code		  */
	#define	kIDTool		'TO'	/*					T O x x ---> SerCom tool function code			  */
	#define	kIDBIOS		'BI'	/*					B I x x ---> BIOS								  */
	#define	kIDStartUp	'ST'	/*					S T x x ---> Start-up code						  */
	#define	kIDLib		'LI'	/*					L I x x ---> LIB								  */
	#define	kIDDebug	'DG'	/*					D G x x ---> Debug								  */
	#define	kIDProcess	'TA'	/*					T A x x ---> Proc								  */

	uint32	*oPtrCODE;			/* ptr on the CODE segment  */
	uint32	oLnCODE;			/* lenght of the CODE segment */
	uint32	*oPtrDATA;			/* ptr on the DATA segment  */
	uint32	oLnDATA;			/* lenght of the DATA segment */
	uint32	*oPtrStrAppl;		/* ptr on the string	 */
	uint32	oRevision;			/* revision	  */
	uint32	*oPtrStrHelp;		/* ptr on the help string  */
	uint32	oFlType;			/* type of module  */
	#define	BSHOW			0	/* visible in help	  */
	#define	BEXECSERCOM		1	/* executable from SerCom  */
	#define	BEXECRUN		2	/* executable from run	  */
	#define	BCONFIDENTIAL	3	/* no comment		 */

	PROCESS	*oDescProc;			/* ptr on the process descriptor								  	  */
	uint32	oRes2;				/* reserve 2	  	  */
	uint32	oRes1;				/* reserve 1	  	  */
	};

/* Structure of the main big ptr table */
/* ----------------------------------- */

struct	bigPtrTable
	{
	HEADERMODULE	*oPtrFrstMod;		/* ptr on a first module			*/
	uint8			*oPtrTurOTable;		/* ptr on a turret order table		*/
	uint8			*oPtrIOChan;		/* ptr on a I/O channel device		*/
	uint8			*oPtrIDComm;		/* ptr on a ID communication turret	*/
	uint8			*oPtrStUPSys;		/* ptr on a flash start-up system	*/
	uint8			*oPtrStUPUsr;		/* ptr on a flash start-up user		*/
	uint8			*oPtrFrstFreeMe;	/* ptr on a first free memory		*/
	uint8			*oPtrDownLCode;		/* ptr on a downloaded code			*/
	};

/* BIOS system calls */
/* ================= */

/* Special macros & calls (not original BIOS functions) to control the kernel */
/* -------------------------------------------------------------------------- */

#define	RESERVE_SER		while (ser_reserve_channel() == -1) tim_switch_fast();
#define	RELEASE_SER		ser_release_channel();

#define	RESERVE_MSG(channelNb)	while (msg_reserve_channel(channelNb) == -1) tim_switch_fast();
#define	RELEASE_MSG(channelNb)	msg_release_channel(channelNb);

#define	RESERVE_COM		while (com_reserve_channel() == -1) tim_switch_fast();
#define	RELEASE_COM		com_release_channel();

#define RESERVE_USART                   while (usart_reserve_channel() == -1) tim_switch_fast();
#define RELEASE_USART                   usart_release_channel();

#define	RESERVE_MMA0	while (mma_reserve_channel_0() == -1) tim_switch_fast();
#define	RELEASE_MMA0	mma_release_channel_0();

#define	RESERVE_MMA1	while (mma_reserve_channel_1() == -1) tim_switch_fast();
#define	RELEASE_MMA1	mma_release_channel_1();

#define	RESERVE_MMA2	while (mma_reserve_channel_2() == -1) tim_switch_fast();
#define	RELEASE_MMA2	mma_release_channel_2();

#define	DEBUGGER		asm("	trap	#7");

#define INTERRUPTION_SET                asm("   move.w  #$2400,SR");

#define INTERRUPTION_OFF                asm("   move.w  SR,-(SP)");     \
                                                                asm("   move.w  #$2700,SR");

#define INTERRUPTION_RESTORED   asm("   move.w  (SP)+,SR");

/* Not original BIOS system calls */
/* ============================== */

typedef struct task
{
uint32 idTask;
uint32 *stackTask;
}Tasks;

extern  Tasks *get_task_array(void);
#define install_task(textId, stackLength, procedure) install_task_parms(textId, stackLength, procedure, NULL)
extern  int32   install_task_parms(char *textId, uint32 stackLength, void (*procedure)(), void *parm);
extern	int32	kill_task(uint32 id);
extern  int32   tim_new_task(void (*procedure)());

#define CALL_BIOS_0(callNumber) asm("   trap    #0"); \
				asm("   dc.w    "#callNumber"");

#define CALL_BIOS_1(callNumber) asm("   move.l  8(sp),-(sp)"); \
                                asm("   trap    #0"); \
                                asm("   dc.w    "#callNumber""); \
                                asm("   add.l   #1*4,sp");

#define CALL_BIOS_2(callNumber) asm("   move.l  12(sp),-(sp)"); \
				asm("   move.l  12(sp),-(sp)"); \
				asm("   trap    #0"); \
				asm("   dc.w    "#callNumber""); \
				asm("   add.l   #2*4,sp");

#define CALL_BIOS_3(callNumber) asm("   move.l  16(sp),-(sp)"); \
				asm("   move.l  16(sp),-(sp)"); \
				asm("   move.l  16(sp),-(sp)"); \
				asm("   trap    #0"); \
				asm("   dc.w    "#callNumber""); \
				asm("   add.l   #3*4,sp");

#define CALL_BIOS_4(callNumber) asm("   move.l  20(sp),-(sp)"); \
                                asm("   move.l  20(sp),-(sp)"); \
				asm("   move.l  20(sp),-(sp)"); \
				asm("   move.l  20(sp),-(sp)"); \
				asm("   trap    #0"); \
				asm("   dc.w    "#callNumber""); \
				asm("   add.l   #4*4,sp");

#define CALL_BIOS_5(callNumber) asm("   move.l  24(sp),-(sp)"); \
				asm("   move.l  24(sp),-(sp)"); \
                                asm("   move.l  24(sp),-(sp)"); \
                                asm("   move.l  24(sp),-(sp)"); \
                                asm("   move.l  24(sp),-(sp)"); \
                                asm("   trap    #0"); \
                                asm("   dc.w    "#callNumber""); \
                                asm("   add.l   #5*4,sp");
 
/* BIOS 0..49 */
/* ---------- */

extern  void            bios_reset(void);
extern  char            *bios_get_ident(void);
extern  uint32          bios_get_rev(void);
extern  uint32          bios_get_system(void);
extern  void            bios_restart_system(void);
extern  uint32          bios_get_node(void);
extern  int32           bios_restart_launch(uint32 idFunction, uint32 baudrate);
/* TIM 50..99 */
/* ---------- */

extern  void            tim_reset(void);
extern  int32           tim_new_inst_task(char *textId, uint32 *stack, void (*procedure)(), void *parm);
extern  int32           tim_remove_inst_task(uint32 id);
extern  void            tim_suspend_task(uint32 time);
extern  void            tim_generate_event(void);
extern  void            tim_wait_event(uint32 taskMask);
extern  uint32          tim_get_id(void);
extern  uint32          tim_get_ticcount(void);
extern  void            tim_switch_fast(void);
extern  void            tim_lock(void);
extern  void            tim_unlock(void);
extern  int32           tim_define_association(char *reference, uint32 *general);
extern  int32           tim_find_association(char *reference);
extern  int32           tim_remove_association(char *reference);
extern  void            tim_wait_sync(uint32 syncMask);
extern  PROCDESC        *tim_get_task_des_ptr(void);

/* SENS 100..149 */
/* ------------- */

extern  void            sens_reset(void);
extern  int32           sens_get_ana_value(uint32 inputNb);

/* MSG 150..199 */
/* ------------ */

extern  void            msg_reset(void);
extern  int32           msg_reserve_channel(uint32 channelNb);
extern  int32           msg_release_channel(uint32 channelNb);
extern  int32           msg_send_message(uint8 *mesgS, uint32 sizeS);
extern  int32           msg_receive_message(uint8 *mesgR, uint32 sizeR);
extern  int32           msg_snd_rec_message(uint8 *mesgS, uint32 sizeS, uint8 *mesgR, uint32 sizeR, uint32 rep);

/* VAR 200..249 */
/* ------------ */

extern  void            var_reset(void);
extern  uint32          var_get_jumper(void);
extern  int32           var_on_led(uint32 ledNb);
extern  int32           var_off_led(uint32 ledNb);
extern  int32           var_change_led(uint32 ledNb);
extern  void            var_set_irq_vector(void(*procedure)());
extern  void            var_enable_irq(void);
extern  void            var_disable_irq(void);
extern  int32           var_set_exception(void(*procedure)(), uint32 exceptionNb);
extern  int32           var_cpu_speed(uint32 CPUSpeedValue);
extern  uint32          var_get_extension(uint32 address);
extern  void            var_put_extension(uint32 address, uint32 binaryValue);

/* SER 250..299 */
/* ------------ */

extern  void            ser_reset(void);
extern  int32           ser_reserve_channel(void);
extern  void            ser_release_channel(void);
extern  int32           ser_config(uint32 baudrate);
extern  int32           ser_send_buffer(uint8 *buffer, uint32 size);
extern  int32           ser_receive_byte(void);
extern  int32           ser_tx_status(void);
extern  int32           ser_rx_status(void);

/* STR 300..349 */
/* ------------ */
extern  void            str_reset(void);
extern  int32           str_cnvt_dascii_bin32(uint32 *binary, char *ascii, uint32 paramNb);
extern  int32           str_cnvt_dascii_bin8(uint8 *binary, char *ascii, uint32 paramNb);
extern  void            str_cnvt_bin32_dascii(char *ascii, uint32 *binary,uint32 paramNb);
extern  void            str_cnvt_bin8_dascii(char *ascii, uint8 *binary, uint32 paramNb);
extern  uint32          str_get_size_ascii(char *ascii);                   extern  int32           str_skip_param_protocol(char *ascii, uint32 paramNb
);
extern  char            *str_cnvt_vbin_dascii(char *ascii, uint32 binaryValue);
extern  int32           str_skip_param(char *ascii, uint32 paramNb);
extern  char            *str_cnvt_vbin_hascii(char *ascii, uint32 binaryValue);
extern  int32           str_cnvt_hascii_bin32(uint32 *binary, char *ascii, uint32 paramNb);
extern  int32           str_cnvt_hascii_bin8(uint8 *binary, char *ascii, uint32 paramNb);
extern  void            str_cnvt_bin32_hascii(char *ascii, uint32 *binary,
uint32 paramNb);
extern  void            str_cnvt_bin8_hascii(char *ascii, uint8 *binary, uint32 paramNb);

/* MMA 350..399 */
/* ------------ */

extern  void            mma_reset(void);
extern  int32           mma_reserve_channel_0(void);
extern  int32           mma_reserve_channel_1(void);
extern  int32           mma_reserve_channel_2(void);
extern  void            mma_release_channel_0(void);
extern  void            mma_release_channel_1(void);
extern  void            mma_release_channel_2(void);
extern  int32           mma_send_buffer_0(uint8 *buffer, uint32 size);
extern  int32           mma_send_buffer_1(uint8 *buffer, uint32 size);
extern  int32           mma_send_buffer_2(uint8 *buffer, uint32 size);
extern  int32           mma_receive_byte_0(void);
extern  int32           mma_receive_byte_1(void);
extern  int32           mma_receive_byte_2(void);
extern  int32           mma_tx_status_0(void);
extern  int32           mma_tx_status_1(void);
extern  int32           mma_tx_status_2(void);
extern  int32           mma_rx_status_0(void);
extern  int32           mma_rx_status_1(void);
extern  int32           mma_rx_status_2(void);

/* USART 400..449 */
/* -------------- */

extern  void            usart_reset(void);
extern  int32           usart_reserve_channel(void);
extern  void            usart_release_channel(void);
extern  int32           usart_config(uint32 baudrate);
extern  int32           usart_send_buffer(uint8 *buffer, uint32 size);
extern  int32           usart_receive_byte(void);
extern  int32           usart_tx_status(void);
extern  int32           usart_rx_status(void);

/* MOT 450..499 */
/* ------------ */

extern  void    mot_reset(void);
extern  int32   mot_config_speed_1m(uint32 motorNb, uint32 kp, uint32 ki, uint32 kd);
extern  int32   mot_new_speed_1m(uint32 motorNb, int32 speed);
extern  int32   mot_get_position(uint32 motorNb);
extern  int32   mot_get_speed(uint32 motorNb)   ;
extern  int32   mot_put_sensors_1m(uint32 motorNb, int32 position);
extern  int32   mot_stop(void)  ;
extern  int32   mot_new_position_1m(uint32 motorNb, int32 position);
extern  int32   mot_new_pwm_1m(uint32 motorNb, int32 pwm)       ;
extern  int32   mot_new_speed_2m(int32 speed1, int32 speed0);
extern  int32   mot_config_position_1m(uint32 motorNb, uint32 kp, uint32 ki, uint32 kd);
extern  int32   mot_put_sensors_2m(int32 position1, int32 position0);
extern  int32   mot_new_position_2m(int32 position1, int32 position0);
extern  int32   mot_config_profil_1m(uint32 motorNb, uint32 maxSpeed, uint32 maxAcceleration);
extern  int32   mot_get_status(uint32 motorNb);
extern  int32   mot_new_pwm_2m(int32 pwm1, int32 pwm0);
extern  int32   mot_config_system(int32 motorNb, int32 ConsignMode, int32 CommandMode, int32 FeedBackMode);

/* EXT 500..549 */
/* ------------ */

extern  void    ext_reset(void);
extern  int32   ext_get_sens_value(uint32 sensorNb);
extern  int32   ext_put_analog_value(uint32 DACnum, int32 DACvalue);
extern  void    ext_new_IO_configuration(uint32 IOconfiguration);
extern  int32   ext_new_pin_IO_configuration(uint32 pinNumber, uint32 pinState);

extern  void    ext_new_IO_value(uint32 value);
extern  int32   ext_get_IO_value(void);
extern  int32   ext_new_IO_pin_value(uint32 numBit, uint32 state);
extern  void    ext_new_power_out(uint32 powerValue);
extern  int32   ext_new_power_out_pin(uint32 numBit, uint32 state);

extern  int32   ext_get_version(void);

extern  int32   ext_config_I2C_pins(int32 pinSCL, int32 pinSDA);
extern  int32   ext_write_I2C(int32 i2cAddress, int32 size, const void* buffer);
extern  int32   ext_read_I2C(int32 i2cAddress, int32 size, void* buffer);
extern  void    ext_set_I2C_baudrate(uint32 baudrate);

extern  void    ext_enable_sens(void);
extern  void    ext_disable_sens(void);

extern  int32    ext_config_servo(uint32 numServo, uint32 MinTime, uint32 MaxTime, uint32 polarity); 
extern  int32    ext_new_servo_position(uint32 numServo, uint32 servoPosition);

/* COM system calls */
/* ================ */

#define CALL_COM_0(callNumber)  asm("   trap    #6"); \
                                asm("   dc.w    "#callNumber"");

#define CALL_COM_1(callNumber)  asm("   move.l  8(sp),-(sp)"); \
                                asm("   trap    #6"); \
                                asm("   dc.w    "#callNumber""); \
                                asm("   add.l   #1*4,sp");

#define CALL_COM_2(callNumber)  asm("   move.l  12(sp),-(sp)"); \
                                asm("   move.l  12(sp),-(sp)"); \
				asm("   trap    #6"); \
                                asm("   dc.w    "#callNumber""); \
                                asm("   add.l   #2*4,sp");

#define CALL_COM_3(callNumber)  asm("   move.l  16(sp),-(sp)"); \
                                asm("   move.l  16(sp),-(sp)"); \
				asm("   move.l  16(sp),-(sp)"); \
                                asm("   trap    #6"); \
				asm("   dc.w    "#callNumber""); \
                                asm("   add.l   #3*4,sp");

#define CALL_COM_4(callNumber)  asm("   move.l  20(sp),-(sp)"); \
                                asm("   move.l  20(sp),-(sp)"); \
                                asm("   move.l  20(sp),-(sp)"); \
                                asm("   move.l  20(sp),-(sp)"); \
                                asm("   trap    #6"); \
				asm("   dc.w    "#callNumber""); \
                                asm("   add.l   #4*4,sp");

#define CALL_COM_5(callNumber)  asm("   move.l  24(sp),-(sp)"); \
                                asm("   move.l  24(sp),-(sp)"); \
                                asm("   move.l  24(sp),-(sp)"); \
                                asm("   move.l  24(sp),-(sp)"); \
                                asm("   move.l  24(sp),-(sp)"); \
                                asm("   trap    #6"); \
                                asm("   dc.w    "#callNumber""); \
                                asm("   add.l   #5*4,sp");

/* COM 0..15 */
/* --------- */

extern  void            com_reset(void);
extern  int32           com_reserve_channel(void);
extern  void            com_release_channel(void);
extern  int32           com_send_buffer(uint8 *buffer, uint32 size);
extern  int32           com_receive_byte(void);
extern  int32           com_get_status_channel(void);

#ifdef  __cplusplus
}
#endif

#endif
